"""
Ovde je predstavljena notacija comprehension-a, koja se koristi za kompaktniji zapis for petlji, koje samo sluze
da na neki nacin transformisu nekakvu zadatu listu, kao sto je navedeno u primerima:
"""
lista = [1, 2, 3, 4]
dupla = [2*num for num in lista]
# Izraz 2*num se racuna, tako sto num uzima vrednosti svakog od elemenata liste ponaosob.
# Detalji i primeri:
# http://docs.python.org/2/tutorial/datastructures.html#list-comprehensions

# U sustini, comprehension (sadrzaj u uglastim zagradama) je izraz koji govori kako se formira element nove liste, tj
# govori o transformaciji koja kreira element nove liste - (2*num), i koja se lista koristi kao polazna (for num in lista)
# Ukoliko se comprehension navede unutar uglastih zagrada - [...], citav izraz [2*num for num in lista] ce biti lista,
# kao sto naredne linije govore.
print (dupla)
print (type(dupla))

"""
NAPOMENA: materija izlozena nadalje su generatori

Vise detalja:
 http://anandology.com/python-practice-book/iterators.html#generator-expressions,
 http://stackoverflow.com/questions/102535/what-can-you-use-python-generator-functions-for

"""
# Ako se ne navedu [...], vec oble zagrade - (...), tada je citav izraz u stvari tzv. generator-izraz (generator expression),
# koji pravi jedan po jedan element nove liste, na osnovu transformacije i liste iz koje poticu originalni elementi,
# Sledeci kod ilustruje to:

print("Sa generatorom:")
duplagen=(2*num for num in lista)
print (duplagen)
print (type(duplagen))

# Generator se moze koristiti kao deo for petlje:
for i in duplagen:
    print (i) # svaki put kad se izvrsi petlja, i je zapravo vrednost koja se dobija transformacijom elementa polazne liste
    # jer vrednost za i generise duplagen

# Ako polazna lista ima 100000 clanova, a zelimo da ispisemo sve clanove, duplirane u datoteku duplirani.txt,
# sta ce da bude efikasnije:
# datoteka.writelines([2*num for num in lista])
# ili cemo staviti
# datoteka.writelines(2*num for num in lista)
# hint: sa [] se formira cela lista, a bez toga, ne.
# todo: isprobati jednu i drugu varijantu
